#!/usr/bin/python
###################################
# Stereo echo demonstrating the SndRTThread class
#
#  VL, 01/07

from sndobj import *
import time
import sys

if len(sys.argv) > 1:
    dur = sys.argv[1]
else:
    dur = 60

# SndRTThread object has its own IO objects
# by the default it is created wth 2 channels
t = SndRTThread(2)
# Echo objects take input from SndRTThread inputs
comb_left =  Comb(0.48, 0.6, t.GetInput(1))
comb_right = Comb(0.52, 0.6, t.GetInput(1))
# We add the echo objects to the output channels
t.AddOutput(1, comb_left)
t.AddOutput(2, comb_right)
t.Direct(1)
t.Direct(2)
# Processing
t.ProcOn()
time.sleep(float(dur))
t.ProcOff()
time.sleep(0.1)



#!/usr/bin/python
#####################################
#  spectrum example
#  
#   VL, 01/07

from sndobj import *
from Tkinter import *
import array
import display

# window size, refresh interval, norm factor
# highest frequency of display
window_size = 300
time_interval = 0.1
norm = 32768.0/10
highest = 10000

# display callback
def callb(data):
    re  = array.array('f')
    im  = array.array('f')
    fft  = data[0]
    disp = data[1]
    sr = fft.GetSr()
    end = int(fft.GetFFTSize()*2*highest/sr)
    for i in range(0,end,2):
         re.append(fft.Output(i)/norm)
         im.append(fft.Output(i+1)/norm)
    if not disp.error:
      disp.draw(re,im,(sr/fft.GetHopSize())*time_interval)
    
# SndObj chain and process thread       
win = HammingTable(1024,0.5)
thread = SndThread()
inp = SndRTIO(1, SND_INPUT)
sig = SndIn(inp, 1)
fft = FFT(win,sig);

# display object
disp = display.Spectrum(Tk(), window_size, thread.ProcOff, "red", "blue")
dat = (fft,disp)

# thread set-up
thread.SetProcessCallback(callb, dat)
thread.AddObj(inp, SNDIO_IN)
thread.AddObj(sig)
thread.AddObj(fft)
thread.ProcOn()

# run the display
disp.mainloop()








#!/usr/bin/python
##########################################
#
#  Example using python callbacks
#
#  VL, 01/07

from sndobj import *
import time
import sys

class test:
    def get(self):
       return self.i
    def set(self,val):
        self.i = val
    def __init__(self,val):
       self.i = val

def callb(data):
    data[0].SetFreq(data[1].get(),data[2])
    data[1].set(data[1].get()+1)

def callc(data):
    data[0].SetFreq(data[1].get(),data[2])
    data[1].set(data[1].get()-1)

t = SndThread()
harm = HarmTable(10000,1,1)
osc = Oscili(harm, 440, 10000)
mod = Oscili(harm, 2, 5)
osc.SetFreq(440, mod)
tes = test(100)
dat = (osc, tes, mod)
if sys.platform[:6] == "darwin":
  outp = SndCoreAudio(2, 2048)
  outp.SetOutput(1, osc)
  outp.SetOutput(2, osc)
else:
  outp = SndRTIO(osc)

inp = SndRTIO(1, SND_INPUT, 2048,4)
t.SetProcessCallback(callb, dat)
t.AddObj(osc)
t.AddObj(mod)
t.AddObj(outp, SNDIO_OUT)
t.AddObj(inp, SNDIO_IN)
t.ProcOn()
time.sleep(2)
t.LimitVectorSize(64)

tes.set(100)
time.sleep(2)

t.SetProcessCallback(callc,dat)
tes.set(1000)
t.RestoreVectorSize()
time.sleep(2)
t.ProcOff()
time.sleep(2)





#!/usr/bin/python
###############################################
# A little toy demonstrating PySndObj
#

from Tkinter import *
import sndobj
import math

# set the IO buffersize here
BUFFERSIZE = 2048

# this is our computer instrument
# basically an oscillator and a filter
class Instrument:

    def Output(self):
       sig = self.fil.Output(0)
       return abs(sig)

    def SetFreq(self,freq):
       self.osc.SetFreq(freq*500)

    def SetAmp(self,amp):
       self.amp = amp*5000
       self.osc.SetAmp(self.amp)
       self.fil.SetFreq(self.amp*0.5, None)

    def On(self):
       self.env.SetCurve(0,self.amp)
       self.env.Restart()

    def Off(self):
       self.env.SetCurve(self.amp, 0)
       self.osc.SetAmp(0, self.env)
       self.env.Restart() 

    def __init__(self):
        self.amp = 5000
        self.tab = sndobj.HarmTable(16384, 25, sndobj.SQUARE)
        self.env = sndobj.Interp(0, 0, 0.02)
        self.osc = sndobj.Oscili(self.tab, 440, 0, None, self.env)
        self.fil = sndobj.Lp(100,0.5,self.osc)
        self.outp = sndobj.SndRTIO(1, sndobj.SND_OUTPUT, BUFFERSIZE)
        self.outp.SetOutput(1, self.fil)
        self.thread = sndobj.SndThread(0, None, self.outp)
        self.thread.AddObj(self.env)
        self.thread.AddObj(self.osc)
        self.thread.AddObj(self.fil)
        self.thread.ProcOn()

class Application(Frame):
  
    def move(self,event):
        canvas = event.widget
        x = canvas.canvasx(event.x)
        y = canvas.canvasy(event.y)
        item = canvas.find_withtag("current")[0]
        canvas.coords(item, x+10, y+10, x-10, y-10)
        self.ins.SetFreq(2.0*x/self.size)
        self.ins.SetAmp(2.0*(self.size-y)/self.size)
     
    def play(self,event):
        note = event.widget.find_withtag("current")[0]
        for i in range(1,len(self.notes),2):
            if note == self.notes[i]:
                val = self.notes[i-1]
                self.canvas.itemconfigure(i, fill="red")
                self.ins.On()
                self.ins.SetFreq(val)

    def stop(self,event):
        self.on = False
        note = event.widget.find_withtag("current")[0]
        for i in range(1,len(self.notes),2):
            if note == self.notes[i]:
                val = self.notes[i-1]
                self.ins.Off()
                
    def createCanvas(self):
        self.size = 600
        self.canvas = Canvas(self,height=self.size,width=self.size,bg="violet")
        self.canvas.pack()
    
         
    def createShapes(self):
          item = self.canvas.create_oval(self.size/2-10,self.size/2-10,self.size/2+10,self.size/2+10,fill="black")
          self.items.append(item)
          self.notes.append(440)
          self.notes.append(item)
          self.canvas.tag_bind(item,"<ButtonPress>", self.play)
          self.canvas.tag_bind(item,"<B1-Motion>", self.move)
          self.canvas.tag_bind(item,"<ButtonRelease>", self.stop)

        
    def meter(self):
        iw = 10
        for i in range(0, self.size, iw):
            self.vu.append(self.canvas.create_rectangle(i,self.size-40,i+iw,self.size,fill="grey"))

    def draw(self):
        level = self.ins.Output()
        cnt = 0
        red = (self.size/10)*0.8
        yellow = (self.size/10)*0.6
        for i in self.vu:
          if level > cnt*200:
           if cnt > red:
            self.canvas.itemconfigure(i, fill="red")
           elif cnt > yellow:
            self.canvas.itemconfigure(i, fill="yellow")
           else:
            self.canvas.itemconfigure(i, fill="blue")
          else:
           self.canvas.itemconfigure(i, fill="grey")
          cnt  = cnt + 1
        self.master.after(50,self.draw)
                
    def quit(self):
        self.master.destroy()

    def __init__(self,master=None):
        master.title("PySndObj + Tkinter: just click and play")
        self.items = []
        self.notes = []
        Frame.__init__(self,master)
        self.pack()
        self.createCanvas()  
        self.createShapes()
        self.ins = Instrument()
        self.vu = []
        self.meter()
        master.after(100, self.draw),
        self.master = master
        self.master.protocol("WM_DELETE_WINDOW", self.quit)

app = Application(Tk())
app.mainloop()



#!/usr/bin/python
#####################################
#  oscilloscope example
#  
#   VL, 01/07

from sndobj import *
from Tkinter import *
import display
import array

# window size, refresh interval and norm factor
window_size = 300
time_interval = 0.1
norm = 32768.0

# display callback
def callb(data):
    sig  = array.array('f')
    osc  = data[0]
    disp = data[1]
    for i in range(0,osc.GetVectorSize()):
        sig.append(osc.Output(i)/norm)
    disp.draw(sig,time_interval*osc.GetSr())

# SndObj chain and process thread       
harm = HarmTable(10000,1,1)
mod = Oscili(harm, 10, 10000)
osc = Oscili(harm, 400, 0)
outp = SndRTIO(1, SND_OUTPUT, 4096)
outp.SetOutput(1, osc)
thread = SndThread()

# display object
master = Tk()

disp = display.Oscilloscope(master, window_size, thread.ProcOff, "green", "black")
dat = (osc,disp)

# thread set-up
thread.SetProcessCallback(callb, dat)
thread.AddObj(mod)
thread.AddObj(osc)
thread.AddObj(outp, SNDIO_OUT)
thread.ProcOn()

# switch the sound after 2.5 s
master.after(2500, osc.SetAmp,10000,mod)

# run the display
disp.mainloop()








from sndobj import SndObj, SndRTIO, HarmTable, Oscili, SND_OUTPUT
from scipy import zeros, pi, sin, float32
from pylab import plot, show
    
# ---------------------------------------------------------------------------
# Test PushIn

# Create 1 frame of a sine wave in a numpy array
sine = zeros(256, dtype=float32)
for i in range(sine.size):
    sine[i] = 0.5 * sin((2 * pi * i) / sine.size)
sine *= 32768

obj = SndObj()
obj.PushIn(sine)

outp = SndRTIO(1, SND_OUTPUT)
outp.SetOutput(1, obj)

# Repeatedly output the 1 frame of sine wave
duration = outp.GetSr() * 2  # 2 seconds
i = 0
vector_size = outp.GetVectorSize()
while i < duration:
    outp.Write()
    i += vector_size

# ---------------------------------------------------------------------------    
# Test PopOut

# copy a sine wave generated by a SndObj into a numpy array
output_size = 2048
harm = HarmTable(10000,1,1)
osc = Oscili(harm, 440, 10000)
output = zeros(output_size, dtype=float32)

duration = output_size  # output_size samples
i = 0
vector_size = osc.GetVectorSize()
while i < duration:
    osc.DoProcess()
    osc.PopOut(output[i:i+vector_size])
    i += vector_size

# plot the array
plot(output)
show()



from distutils.core import setup

setup(name='SndObj',
      version='2.6.99',
      description='SndObj Library',
      author='Victor Lazzarini',
      author_email='Victor.Lazzarini@nuim.ie',
      url='http://sndobj.sourceforget.net',
      packages=['sndobj'],
      package_data={'sndobj': ['*.pyd', '*.dll', '*.lib', '*.exp']},
     )




###################################
# PySndObj extensions: display classes
#
# VL, 01/07

from Tkinter import *
import time
import math
import array

class Oscilloscope(Frame):
                
    def createCanvas(self):
        self.canvas = Canvas(self,height=self.size,width=self.size,bg=self.bg)
        self.canvas.pack()

    def draw(self,what,samples):
        for i in what:
           self.data.append(i);
        s = len(self.data)
        if s >= samples:
            count = 0
            for i in self.item:
               self.canvas.delete(i)
            del self.item[0:len(self.item)]
            incr = s/self.size
            j = 0
            while(j < s):
              y = self.data[j]*self.size/2 + self.size/2
              self.item.append(self.canvas.create_line(count, self.prev, count+1, y, fill=self.line))
              self.prev = y       
              count += 1
              j += incr
            del self.data[0:s]
      
    def quit(self):
        self.close();
        time.sleep(0.1)
        self.master.destroy()

    def __init__(self,master, size, end, line="black", bg="white"):
        master.title("Oscilloscope")
        self.size = size
        self.close = end
        self.line = line
        self.bg = bg
        self.data = array.array('f')
        Frame.__init__(self,master)
        self.pack()
        self.createCanvas()
        self.item = array.array('i')
        self.prev = 0
        self.n = 0
        self.master = master
        self.master.protocol("WM_DELETE_WINDOW", self.quit)

class Spectrum(Oscilloscope):

      def draw(self,re,im,frames):
         s = len(re)
         for i in range(0, s):
            y1 = re[i]
            y2 = im[i]
            if self.cnt == 0:
              self.spec.append(y1*y1+y2*y2)
            else:
              self.spec[i] = (y1*y1+y2*y2+self.spec[i])
         self.cnt += 1
         if self.cnt >= frames:
            self.cnt = 0
            count = 0
            for i in self.item:
               self.canvas.delete(i)
            del self.item[0:len(self.item)]
            s = len(self.spec)
            j = 0.0
            incr = (float(s)/self.size)
            while(j < s):
              y = -math.sqrt(self.spec[int(j)]/frames)*self.size + self.size 
              self.item.append(self.canvas.create_line(count, self.prev, count+1, y, fill=self.line, width=self.width))
              self.prev = y       
              count += 1
              j += incr
            del self.spec[0:s]


      def __init__(self,master,size,end,line="black",bg="white",width=1):
          Oscilloscope.__init__(self,master,size,end,line,bg)
          master.title("Spectrum")
          self.cnt = 0
          self.spec = array.array('f')
          self.error = 0
          self.width = width
          



from sndobj import *
import time
import sys

class test:
    def get(self):
       return self.i
    def set(self,val):
        self.i = val
    def __init__(self,val):
       self.i = val

def callb(data):
    data[0].SetFreq(data[1].get(),data[2])
    data[1].set(data[1].get()+1)

def callc(data):
    data[0].SetFreq(data[1].get(),data[2])
    data[1].set(data[1].get()-1)

t = SndRTThread()

harm = HarmTable(10000,1,1)
osc = Oscili(harm, 440, 10000)
mod = Oscili(harm, 2, 5)
osc.SetFreq(440, mod)
tes = test(100)
dat = (osc, tes, mod)

t.SetProcessCallback(callb, dat)
t.AddObj(mod)
t.AddOutput(1, osc)

t.ProcOn()
time.sleep(2)
t.LimitVectorSize(64)
tes.set(100)

time.sleep(2)
t.SetProcessCallback(callc,dat)
tes.set(1000)
t.RestoreVectorSize()
time.sleep(2)
t.ProcOff()




